@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio HB
+ * Copyright (C) 2003-2004 Imendio AB
  * Copyright (C) 2001-2003 CodeFactory AB
  * Copyright (C) 2001-2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
@@ -24,10 +24,8 @@
 
 #include <config.h>
 #include <string.h>
-#include <gsf/gsf-input-memory.h>
-#include <gsf/gsf-input-stdio.h>
 #include "mrp-error.h"
-#include "mrp-intl.h"
+#include <glib/gi18n.h>
 #include "mrp-marshal.h"
 #include "mrp-task-manager.h"
 #include "mrp-storage-module-factory.h"
@@ -413,7 +411,6 @@ static void
 project_init (MrpProject *project)
 {
 	MrpProjectPriv *priv;
-	MrpProperty    *property;
 	MrpCalendar    *calendar;
 	
 	project->priv = g_new0 (MrpProjectPriv, 1);
@@ -446,7 +443,7 @@ project_init (MrpProject *project)
 		(GDestroyNotify) mrp_day_unref);
 	
         /* Move these into the cost plugin */
-	property = mrp_property_new ("cost", 
+/*	property = mrp_property_new ("cost", 
 				     MRP_PROPERTY_TYPE_COST,
 				     "Cost",
 				     "standard cost for a resource",
@@ -455,7 +452,7 @@ project_init (MrpProject *project)
 				  MRP_TYPE_RESOURCE,
 				  property,
                                   TRUE);
-
+*/
 	/* FIXME: Add back this when it's used. */
 /*	property = mrp_property_new ("cost_overtime", 
 	MRP_PROPERTY_TYPE_COST,
@@ -682,9 +679,10 @@ gboolean
 mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 {
 	MrpProjectPriv *priv;
-	GsfInput       *input;
 	GList          *l;
 	MrpCalendar    *old_default_calendar;
+	gchar          *file_str;
+	size_t          len;
 	
 	g_return_val_if_fail (MRP_IS_PROJECT (project), FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
@@ -701,25 +699,38 @@ mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 	 */
 	old_default_calendar = priv->calendar;
 
- 	input = GSF_INPUT (gsf_input_mmap_new (uri, NULL));
-	
- 	if (!input) {
-		input = GSF_INPUT (gsf_input_stdio_new (uri, error));
-	}
 
-	if (!input) {
+	len = strlen (uri);
+
+	/* Get a local file from the uri. */
+	if (len > 3 && !strstr (uri, ":/")) {
+		/* No protocol. */
+	} else {
+		if (len > 7 && !strncmp (uri, "file:/", 6)) {
+			/* Naively strip method. */
+			uri = uri + 7;
+		} else {
+			g_set_error (error, 
+				     MRP_ERROR,
+				     MRP_ERROR_INVALID_URI,
+				     _("Invalid URI: '%s'"),
+				     uri);
+			
+			return FALSE;
+		}
+	}
+	
+	if (!g_file_get_contents (uri, &file_str, NULL, error)) {
 		return FALSE;
 	}
-
+	
 	mrp_task_manager_set_block_scheduling (priv->task_manager, TRUE);
 	
 	l = imrp_application_get_all_file_readers (priv->app);
 	for (; l; l = l->next) {
 		MrpFileReader *reader = l->data;
 
-		if (mrp_file_reader_read (reader, input, project, error)) {
-			g_object_unref (input);
-			
+		if (mrp_file_reader_read_string (reader, file_str, project, error)) {
 			g_signal_emit (project, signals[LOADED], 0, NULL);
 			imrp_project_set_needs_saving (project, FALSE);
 
@@ -730,17 +741,17 @@ mrp_project_load (MrpProject *project, const gchar *uri, GError **error)
 			mrp_calendar_remove (old_default_calendar);
 
 			mrp_task_manager_set_block_scheduling (priv->task_manager, FALSE);
-
-			/* FIXME: See bug #416. */
 			imrp_project_set_needs_saving (project, FALSE);
 			
+			g_free (file_str);
+			
 			return TRUE;
 		}
 	}
 
-	mrp_task_manager_set_block_scheduling (priv->task_manager, FALSE);
+	g_free (file_str);
 
-	g_object_unref (input);
+	mrp_task_manager_set_block_scheduling (priv->task_manager, FALSE);
 
 	g_set_error (error, 
 		     MRP_ERROR,
@@ -1428,6 +1439,21 @@ mrp_project_get_uri (MrpProject *project)
 	return project->priv->uri;
 }
 
+/**
+ * mrp_project_set_uri:
+ * @project: an #MrpProject
+ * 
+ * Sets the URI on @project.
+ **/
+void    
+mrp_project_set_uri (MrpProject *project, const gchar *uri)
+{
+	g_return_if_fail (MRP_IS_PROJECT (project));
+
+	g_free (project->priv->uri);
+	project->priv->uri = g_strdup (uri);
+}
+
 /**
  * mrp_project_get_project_start:
  * @project: an #MrpProject
@@ -1891,6 +1917,35 @@ mrp_project_reschedule (MrpProject *project)
 	mrp_task_manager_recalc (project->priv->task_manager, TRUE);
 }
 
+/**
+ * mrp_project_calculate_summary_duration:
+ * @project: an #MrpProject
+ * @task: an #MrpTask
+ * @start: a start time, or if %-1, the task start time is to be used 
+ * @finish: a finish time
+ * 
+ * Calculates the time needed to achieve the given start and finish time, 
+ * ignoring allocated resources' calendars (summary tasks should ignore
+ * their own resource assignments)
+ * 
+ * Return value: The calculated duration.
+ **/
+gint
+mrp_project_calculate_summary_duration (MrpProject *project, 
+				 MrpTask    *task, 
+				 mrptime     start,
+				 mrptime     finish)
+{
+	g_return_val_if_fail (MRP_IS_PROJECT (project), 0);
+	g_return_val_if_fail (MRP_IS_TASK (task), 0);
+	g_return_val_if_fail (start == -1 || start <= finish, 0);
+	g_return_val_if_fail (finish >= 0, 0);
+
+	return mrp_task_manager_calculate_summary_duration (
+		project->priv->task_manager,
+		task, start, finish);
+}
+
 /**
  * mrp_project_calculate_task_work:
  * @project: an #MrpProject
@@ -2259,3 +2314,28 @@ project_set_calendar (MrpProject  *project,
 
 	mrp_task_manager_recalc (priv->task_manager, TRUE);
 }
+
+void
+mrp_project_set_block_scheduling (MrpProject *project, gboolean block)
+{
+	MrpProjectPriv *priv;
+
+	g_return_if_fail (MRP_IS_PROJECT (project));
+
+	priv = project->priv;
+
+	mrp_task_manager_set_block_scheduling (priv->task_manager, block);
+}
+
+gboolean
+mrp_project_get_block_scheduling (MrpProject *project)
+{
+	MrpProjectPriv *priv;
+
+	g_return_val_if_fail (MRP_IS_PROJECT (project), FALSE);
+
+	priv = project->priv;
+	
+	return mrp_task_manager_get_block_scheduling (priv->task_manager);
+}
+
