@@ -23,7 +23,8 @@
 #include <config.h>
 #include <string.h>
 #include <glib.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include "libplanner/mrp-paths.h"
 #include "planner-plugin.h"
 #include "planner-plugin-loader.h"
 
@@ -34,7 +35,7 @@ mpl_load (const gchar *file)
 
 	plugin = g_object_new (PLANNER_TYPE_PLUGIN, NULL);
 	
-	plugin->handle = g_module_open (file, G_MODULE_BIND_LAZY);
+	plugin->handle = g_module_open (file, 0);
 	
 	if (plugin->handle == NULL) {
 		g_warning (_("Could not open plugin file '%s'\n"),
@@ -43,8 +44,8 @@ mpl_load (const gchar *file)
 		return NULL;
 	}
 	
-	g_module_symbol (plugin->handle, "plugin_init", (gpointer*)&plugin->init);
-	g_module_symbol (plugin->handle, "plugin_exit", (gpointer*)&plugin->exit);
+	g_module_symbol (plugin->handle, "plugin_init", (gpointer)&plugin->init);
+	g_module_symbol (plugin->handle, "plugin_exit", (gpointer)&plugin->exit);
 
 	return plugin;
 }
@@ -63,11 +64,12 @@ mpl_load_dir (const gchar *path, PlannerWindow *window)
 	}
 	
 	while ((name = g_dir_read_name (dir)) != NULL) {
-		if (strncmp (name + strlen (name) - 3, ".so", 3) == 0) {
-			gchar *plugin_name =  g_build_path (G_DIR_SEPARATOR_S,
-							    path,
-							    name,
-							    NULL);
+		if (g_str_has_suffix (name, G_MODULE_SUFFIX)) {
+			gchar *plugin_name;
+
+			plugin_name = g_build_filename (path,
+							name,
+							NULL);
 			plugin = mpl_load (plugin_name);
 			if (plugin) {
 				list = g_list_append (list, plugin);
@@ -87,5 +89,12 @@ mpl_load_dir (const gchar *path, PlannerWindow *window)
 GList *
 planner_plugin_loader_load (PlannerWindow *window)
 {
-	return mpl_load_dir (MRP_PLUGINDIR, window);
+	gchar  *path;
+	GList  *list;
+	
+	path = mrp_paths_get_plugin_dir (NULL);
+	list = mpl_load_dir (path, window);
+	g_free (path);
+	
+	return list;
 }
